@@ -1,6 +1,6 @@
 #include <stdio.h>
 
-static char version[] = "0.8.14 beta";
+static char version[] = "0.9.0 beta";
 
 // 関数プロトタイプ
 void help(void);
@@ -41,6 +41,12 @@ void help()
 //	puts("");
 //	puts(" -d         ダイレクトI/Oを使用する。");
 //	puts("            (このオプションは非推奨です。)");
+	puts("");
+	puts(" -h         ヘルプを表示する。");
+	puts("            (例 : snowcp -h)");
+	puts("");
+	puts(" --help     ヘルプを表示する。");
+	puts("            (例 : snowcp --help)");
 	puts("");
 	puts(" -i         ファイルを上書きする前に確認する。");
 	puts("            (例 : snowcp -i source dest)");
