@@ -95,7 +95,6 @@ static long psize = 0;
 static _Bool from_open;
 static _Bool from_stat;
 static _Bool to_open;
-//static _Bool filesystem_is_ext4;
 
 // ※注意。グローバル変数。
 char *write_now;
@@ -110,11 +109,12 @@ static void * write_thread(void *unknown);
 _Bool read_write(const OPArg *oparg, CPInfo *cpinfo, SDir *sdir, CPDD *cpdd)
 {
 	static struct stat stat_buf;
+	static _Bool filesystem_check = false;
+	static _Bool filesystem_is_fallocate_support = false;
 
 	from_stat = false;
 	from_open = false;
 	to_open = false;
-//	filesystem_is_ext4 = false;
 	from_size_local = 0;
 	written_size[0] = 0;
 	written_size[1] = 0;
@@ -212,56 +212,56 @@ _Bool read_write(const OPArg *oparg, CPInfo *cpinfo, SDir *sdir, CPDD *cpdd)
 		}
 	}
 
-	/*
-	 * posix_fallocateはext4以外の、それをサポートしていないファイルシステムだとかなり遅くなる様子。
-	 * ext4だと4秒弱で終わるフォルダのコピーが、ntfsだと12秒はかかる。
-	 * （何もしないと6秒弱、pwriteだと8秒ほど、ftruncateは何もしない場合とほぼ同じ）
-	*/
-	if((from_size_local > 0) && (to_open == true))
+	if(filesystem_check == false)
 	{
 		char *tofs_tmp = cpinfo->tofs;
 
-		if(
-		(tofs_tmp != NULL) &&
-		(tofs_tmp[0] == 'e') &&
-		(tofs_tmp[1] == 'x') &&
-		(tofs_tmp[2] == 't') &&
-		(tofs_tmp[3] == '4')
-		)
+		if(tofs_tmp != NULL)
 		{
-			static int itmp = -1;
-//			filesystem_is_ext4 = true;
-
-			// Ubuntu 11.04だとfallocateの第二引数は FALLOC_FL_KEEP_SIZE と FALLOC_FL_PUNCH_HOLE の二つ。
-			// linux/falloc.h のincludeが必要。
-			if((itmp = fallocate(ito, FALLOC_FL_KEEP_SIZE, 0, from_size_local)) != 0)
+			if
+			(
+				((tofs_tmp[0] == 'e') && (tofs_tmp[1] == 'x') && (tofs_tmp[2] == 't') && (tofs_tmp[3] == '4'))
+				||
+				((tofs_tmp[0] == 'x') && (tofs_tmp[1] == 'f') && (tofs_tmp[2] == 's'))
+				||
+				((tofs_tmp[0] == 'b') && (tofs_tmp[1] == 't') && (tofs_tmp[2] == 'r') && (tofs_tmp[2] == 'f') && (tofs_tmp[2] == 's'))
+				||
+				((tofs_tmp[0] == 'o') && (tofs_tmp[1] == 'c') && (tofs_tmp[2] == 'f') && (tofs_tmp[2] == 's') && (tofs_tmp[2] == '2'))
+			)
 			{
-				errno = itmp;
-				print_error("fallocate", __FILE__, __LINE__, cpdd);
+				filesystem_is_fallocate_support = true;
 			}
-/*
-			if((itmp = posix_fallocate(ito, 0, from_size_local)) != 0)
-			{
-				errno = itmp;
-				print_error("posix_fallocate", __FILE__, __LINE__, cpdd);
-			}
-*/
 		}
-		else
+
+		filesystem_check = true;
+	}
+
+	/*
+	 * ファイルシステムがfallocateをサポートしている場合、それを実行して領域を確保する。
+	 * 
+	 * posix_fallocateはext4以外の、それをサポートしていないファイルシステムだとかなり遅くなる様子。
+	 * ext4だと4秒弱で終わるフォルダのコピーが、
+	 * ntfsだと12秒はかかる（何もしないと6秒弱、pwriteだと8秒ほど、ftruncateは何もしない場合とほぼ同じ）。
+	*/
+	if((from_size_local > 0) && (to_open == true) && (filesystem_is_fallocate_support == true))
+	{
+		static int itmp = -1;
+
+		// Ubuntu 11.04だとfallocateの第二引数は FALLOC_FL_KEEP_SIZE と FALLOC_FL_PUNCH_HOLE の二つ。
+		// linux/falloc.h のincludeが必要。
+		if((itmp = fallocate(ito, FALLOC_FL_KEEP_SIZE, 0, from_size_local)) != 0)
 		{
-			errno = 0;
-
-			if(ftruncate(ito, from_size_local) == -1)
-			{
-				print_error("ftruncate", __FILE__, __LINE__, cpdd);
-			}
-/*
-			if(pwrite(ito, "", 1, from_size_local - 1) == -1)
-			{
-				print_error("pwrite", __FILE__, __LINE__, cpdd);
-			}
-*/
+			errno = itmp;
+			print_error("fallocate", __FILE__, __LINE__, cpdd);
 		}
+
+/*
+		if((itmp = posix_fallocate(ito, 0, from_size_local)) != 0)
+		{
+			errno = itmp;
+			print_error("posix_fallocate", __FILE__, __LINE__, cpdd);
+		}
+*/
 	}
 
 	// コピー開始
@@ -430,10 +430,6 @@ LOOP_END:
 		 * または大きなファイルを小さなファイルで上書きした場合のリサイズ。
 		 * open時にO_TRUNCしてない場合、重要な処理。
 		 * ここを弄ってバージョン0.1.2でバグを出したので、うかつに弄らない。
-		 * 
-		 * 0.4.0から、
-		 * openした後にposix_fallocate、fallocate、ftruncate、pwriteのいずれかを、
-		 * 必ず行うようにしたが、念のためここの処理も残しておくべき。
 		*/
 		{
 			static struct stat to_stat_tmp;
@@ -442,15 +438,12 @@ LOOP_END:
 			{
 				print_error("fstat", __FILE__, __LINE__, cpdd);
 			}
-			else if(from_size_local >= to_stat_tmp.st_size)
+			else if(from_size_local < to_stat_tmp.st_size)
 			{
-			}
-			else if(ftruncate(ito, from_size_local) != -1)
-			{
-			}
-			else
-			{
-				print_error("ftruncate", __FILE__, __LINE__, cpdd);
+				if(ftruncate(ito, from_size_local) == -1)
+				{
+					print_error("ftruncate", __FILE__, __LINE__, cpdd);
+				}
 			}
 		}
 	}
@@ -463,7 +456,7 @@ LOOP_END:
 /*
  * fdatasyncするようにしたら、30倍遅くなった。
 
-	if((oparg->C != NOT) && (filesystem_is_ext4 == true))
+	if((oparg->C != NOT) && (filesystem_is_fallocate_support == true))
 	{
 		if(fdatasync(ito) == -1)
 		{
